@@ -1,8 +1,10 @@
+use core::fmt;
 use core::marker;
 
 use crate::integer_encoding::{IntegerEncoding, UsizeEncoding};
 use musli::de::{
-    Decoder, MapDecoder, MapEntryDecoder, PackDecoder, PairDecoder, SequenceDecoder, StructDecoder,
+    Decoder, MapDecoder, MapEntryDecoder, PackDecoder, PairDecoder, ReferenceVisitor,
+    SequenceDecoder, StructDecoder,
 };
 use musli::error::Error;
 use musli_binary_common::reader::Reader;
@@ -77,15 +79,49 @@ where
     }
 
     #[inline]
-    fn decode_bytes(self) -> Result<&'de [u8], Self::Error> {
+    fn decode_bytes<V>(self, visitor: V) -> Result<V::Ok, V::Error>
+    where
+        V: ReferenceVisitor<'de, Target = [u8], Error = Self::Error>,
+    {
         let len = L::decode_usize(&mut *self.reader)?;
-        self.reader.read_bytes(len)
+        let bytes = self.reader.read_bytes(len)?;
+        visitor.visit_ref(bytes)
     }
 
     #[inline]
-    fn decode_str(self) -> Result<&'de str, Self::Error> {
-        let bytes = self.decode_bytes()?;
-        core::str::from_utf8(bytes).map_err(Self::Error::custom)
+    fn decode_string<V>(self, visitor: V) -> Result<V::Ok, V::Error>
+    where
+        V: ReferenceVisitor<'de, Target = str, Error = Self::Error>,
+    {
+        return self.decode_bytes(Visitor(visitor));
+
+        struct Visitor<V>(V);
+
+        impl<'de, V> ReferenceVisitor<'de> for Visitor<V>
+        where
+            V: ReferenceVisitor<'de, Target = str>,
+        {
+            type Target = [u8];
+            type Ok = V::Ok;
+            type Error = V::Error;
+
+            #[inline]
+            fn expected(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+                self.0.expected(f)
+            }
+
+            #[inline]
+            fn visit_ref(self, bytes: &'de [u8]) -> Result<Self::Ok, Self::Error> {
+                let string = core::str::from_utf8(bytes).map_err(Self::Error::custom)?;
+                self.0.visit_ref(string)
+            }
+
+            #[inline]
+            fn visit(self, bytes: &[u8]) -> Result<Self::Ok, Self::Error> {
+                let string = core::str::from_utf8(bytes).map_err(Self::Error::custom)?;
+                self.0.visit(string)
+            }
+        }
     }
 
     #[inline]
